/* $Id: RightClickMenu.java 158 2011-05-08 11:01:20Z ohura $ */
package smart_gs.drawing_tool;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;

import smart_gs.connection.ConnectionCreator;
import smart_gs.dictionary.DictionaryRegisterDialog;
import smart_gs.drawing_tool.action.ChangeBreadthAction;
import smart_gs.drawing_tool.action.ChangeColorAction;
import smart_gs.drawing_tool.action.CloseUpViewerAction;
import smart_gs.drawing_tool.action.CreateShortCutAction;
import smart_gs.drawing_tool.action.DeleteArrowAction;
import smart_gs.drawing_tool.action.DeleteRegionAction;
import smart_gs.drawing_tool.action.MemoPadTextAction;
import smart_gs.drawing_tool.action.NextRegionAction;
import smart_gs.drawing_tool.action.ProtectRegionAction;
import smart_gs.drawing_tool.action.ReleaseProtectedRegionAction;
import smart_gs.drawing_tool.action.SetLinkSourceAction;
import smart_gs.drawing_tool.action.SetLinkTargetAction;
import smart_gs.drawing_tool.action.DrawArrowAction;
import smart_gs.drawing_tool.state.State;
import smart_gs.drawing_tool.view.MemoPadView;
import smart_gs.drawing_tool.view.View;
import smart_gs.logical.Region;
import smart_gs.reasoning_web.logical.GS_RWRepository;
import smart_gs.swingui.WorkspaceWindow;
import smart_gs.swingui.toolbar.action.ShowLocalViewAction;

@SuppressWarnings("serial")
public class RightClickMenu extends JPopupMenu{	
	
	public RightClickMenu(final Region region, SpreadCanvas canvas, final State state) {
		JMenuItem delete = new JMenuItem("Delete Markup");
		delete.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				new DeleteRegionAction(region, state).execute();
			}
		});
		this.add(delete);
		this.addSeparator();
		
		JMenuItem setSource = new JMenuItem("Set Link Source");
		setSource.addActionListener(new SetLinkSourceAction(region));
		this.add(setSource);
		
		JMenuItem setTarget = new JMenuItem("Set Link Target");
		if(ConnectionCreator.isSourceSet()){
			setTarget.setEnabled(true);
	    	setTarget.addActionListener(new SetLinkTargetAction(region));
		}else{
			setTarget.setEnabled(false);
		}
		this.add(setTarget);

		/*
		 *@tsukushi shimizu
		 */
		JMenuItem jump = new JMenuItem("Show Local View");
		if(!GS_RWRepository.getInstance().hasLink(region)){
			jump.setEnabled(false);
		}
		jump.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				new ShowLocalViewAction(region).execute();
			}
		});
		this.add(jump);

		JMenuItem createShortcut = new JMenuItem("Create Short Cut");
		createShortcut.addActionListener(new CreateShortCutAction(region));
		//20080622 shimizu wrote
		//this.add(createShortcut);
		
		//this.addSeparator();
		
		/*
		 *@kazuhiro kobayashi
		 */
		this.addSeparator();
		
		if (region.isProtected().equals(Region.NORMAL)){
			JMenuItem protectRegion = new JMenuItem("Protect Markup");
			protectRegion.addActionListener(new ProtectRegionAction(region));
			this.add(protectRegion);
		} else {
			JMenuItem releaseProtectedRegion = new JMenuItem("Release Protected Markup");
			releaseProtectedRegion.addActionListener(new ReleaseProtectedRegionAction(region));
			this.add(releaseProtectedRegion);
		}
		
		/**
		 *
		 */
		JMenuItem registDic = new JMenuItem("Register in the dictionary");
		registDic.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				DictionaryRegisterDialog dialog = new DictionaryRegisterDialog(WorkspaceWindow.getInstance(), "Register" , true,region);
				dialog.setVisible(true);
			}
		});
		this.add(registDic);

//	2011.01.11 shayashi added
		JMenuItem closeup = new JMenuItem("Close-up");
		if(!(region.getView().getType() == View.RECTANGLE ||
				region.getView().getType() == View.POLYGON)){
			closeup.setEnabled(false);
		}
		closeup.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				new CloseUpViewerAction(region);
			}
		});
		this.add(closeup);

		if (region.getView().getType() == View.MEMOPAD || 
				region.getView().getType() == View.MARKER) {
			
			this.addSeparator();
			JMenu changeColor = new JMenu("Color");
			JMenuItem yellow = new JMenuItem("Yellow");
			yellow.addActionListener(new ChangeColorAction(region, View.ViewColor.YELLOW));
			changeColor.add(yellow);

			JMenuItem cyan = new JMenuItem("Cyan");
			cyan.addActionListener(new ChangeColorAction(region, View.ViewColor.CYAN));
			changeColor.add(cyan);
			
			JMenuItem magenta = new JMenuItem("Magenta");
			magenta.addActionListener(new ChangeColorAction(region, View.ViewColor.MAGENTA));
			changeColor.add(magenta);

			JMenuItem green = new JMenuItem("Green");
			green.addActionListener(new ChangeColorAction(region, View.ViewColor.GREEN));
			changeColor.add(green);
			
			if (region.getView().getType() == View.MEMOPAD) {
				changeColor.addSeparator();
	
				JMenuItem transparent = new JMenuItem("Transparent");
				transparent.addActionListener(new ChangeColorAction(region, View.ViewColor.TRANSPARENT));
				JMenuItem opaque = new JMenuItem("Opaque");
				opaque.addActionListener(new ChangeColorAction(region, View.ViewColor.OPAQUE));
				
				if (((MemoPadView)region.getView()).isTransparent()) {
					changeColor.add(opaque);
				} else {
					changeColor.add(transparent);
				}
			}
			this.add(changeColor);
			if (region.getView().getType() == View.MEMOPAD) {

				JMenuItem addText = new JMenuItem("Add Text");
				addText.addActionListener(new MemoPadTextAction(region));
				this.add(addText);
				
				JMenuItem drawArrow = new JMenuItem("Draw Arrow");
				drawArrow.addActionListener(new DrawArrowAction(region, canvas));
				this.add(drawArrow);
	
				if (((MemoPadView)region.getView()).getArrows().size() != 0) {
					JMenuItem deleteArrow = new JMenuItem("Delete Arrow");
					deleteArrow.addActionListener(new DeleteArrowAction(region, canvas));
					this.add(deleteArrow);
	
				}
			}
			if (region.getView().getType() == View.MARKER) {

				JMenuItem changeBreadth = new JMenuItem("Breadth");
				changeBreadth.addActionListener(new ChangeBreadthAction(canvas, region));
				this.add(changeBreadth);
			}

		}
		this.addSeparator();
		
		View view = region.getView();
		JMenuItem nextRegion = new JMenuItem("Select Next" +" "+ view.getTypeString());
		nextRegion.addActionListener(new NextRegionAction(region, canvas, state));
		this.add(nextRegion);
		

	}
	
}
