/* $Id: ImageToolBar.java 125 2011-03-09 09:49:51Z ohura $ */
package smart_gs.swingui.toolbar;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JToolBar;

import smart_gs.GSConstants;
import smart_gs.drawing_tool.action.DeleteRegionAction;
import smart_gs.drawing_tool.drawing_mode.BookmarkMode;
import smart_gs.drawing_tool.drawing_mode.DefaultDrawingMode;
import smart_gs.drawing_tool.drawing_mode.PolygonMode;
import smart_gs.drawing_tool.drawing_mode.MarkerMode;
import smart_gs.drawing_tool.drawing_mode.MemoPadMode;
import smart_gs.drawing_tool.drawing_mode.RectangleMode;
import smart_gs.drawing_tool.state.DefaultState;
import smart_gs.drawing_tool.state.UniSelectedState;
import smart_gs.logical.Region;
import smart_gs.logical.Sheet;
import smart_gs.swingui.AbstractGSWindow;
import smart_gs.swingui.toolbar.ImageToolBarComponent.IMAGETOOLBARCOMPONENT_TYPE;

@SuppressWarnings("serial")
public class ImageToolBar extends JToolBar implements ActionListener{
	
	private Select select;
	private Eraser eraser;

	private static ModeList modelist;
	public static String[] modelistarray = { "<none>", "Rectangle", "Marker", "Polygon", "MemoPad", "Bookmark" };

	public static int modeNone = 0;
	public static int modeRectangle = 1;
	public static int modeMarker = 2;
	public static int modePolygon = 3;
	public static int modeMemoPad = 4;
	public static int modeBookmark = 5;
	
	private ZoomIn zoomIn;
	private ZoomOut zoomOut;
	
//	private JButton hideRegion;
	private static ShowRegion showRegion;
	public static String[] showregionarray = { "All Regions", "Protected Regions", "No Regions" };
	public static int showAll = 0;
	public static int showProtected = 1;
	public static int showNo = 2;
	
//	private JButton changeSheet;
	
	private AbstractGSWindow parent;

	
	public ImageToolBar(AbstractGSWindow parent){
		this.parent = parent;
		
		this.select = new Select(new ImageIcon(GSConstants.ICON_PATH+"select.gif"));
		this.select.addActionListener(this);
		this.select.setToolTipText("Select");
		this.add(this.select);
		
		this.eraser = new Eraser(new ImageIcon(GSConstants.ICON_PATH+"delete.gif"));
		this.eraser.addActionListener(this);
		this.eraser.setToolTipText("Delete Region");
		this.add(this.eraser);

		modelist = new ModeList(modelistarray);
		modelist.addActionListener(this);
		modelist.setToolTipText("Select Drawing Mode");
		this.add(modelist);
		
		this.zoomIn = new ZoomIn(new ImageIcon(GSConstants.ICON_PATH+"zoom_in.gif"));
		this.zoomIn.addActionListener(this);
		this.zoomIn.setToolTipText("Zoom in");
		this.add(this.zoomIn);
		
		this.zoomOut = new ZoomOut(new ImageIcon(GSConstants.ICON_PATH+"zoom_out.gif"));
		this.zoomOut.addActionListener(this);
		this.zoomOut.setToolTipText("Zoom out");
		this.add(this.zoomOut);
		/*
		 * 2007/9/11
		 */
//		this.hideRegion = new JButton("hideRegion");
//		this.hideRegion.addActionListener(this);
//		this.add(this.hideRegion);
		
		showRegion = new ShowRegion(showregionarray);
		showRegion.addActionListener(this);
		showRegion.setToolTipText("Select the type of Regions to show");
		this.add(showRegion);
//		
//		this.changeSheet = new JButton("Sheet");
//		this.changeSheet.addActionListener(this);
//		this.add(this.changeSheet);
	}

	public void actionPerformed(ActionEvent e) {
		
		if(((ImageToolBarComponent)e.getSource()).getImageToolBarComponentType()
				.equals(IMAGETOOLBARCOMPONENT_TYPE.SELECT)){
			DefaultState.getInstance().setMode(DefaultDrawingMode.getInstance());
			setModeListItem(modelistarray[modeNone]);
		}else if(((ImageToolBarComponent)e.getSource()).getImageToolBarComponentType()
				.equals(IMAGETOOLBARCOMPONENT_TYPE.MODE_LIST)){
			String mode = (String) ((JComboBox) e.getSource()).getSelectedItem();
			if(mode == modelistarray[modeRectangle]) {
				DefaultState.getInstance().setMode(RectangleMode.getInstance());
			} else if(mode == modelistarray[modeMarker]) {
				DefaultState.getInstance().setMode(MarkerMode.getInstance());
			} else if(mode == modelistarray[modePolygon]) {
				DefaultState.getInstance().setMode(PolygonMode.getInstance());
			} else if(mode == modelistarray[modeMemoPad]) {
				DefaultState.getInstance().setMode(MemoPadMode.getInstance());
			} else if(mode == modelistarray[modeBookmark]) {
				DefaultState.getInstance().setMode(BookmarkMode.getInstance());
			}
		}else if(((ImageToolBarComponent)e.getSource()).getImageToolBarComponentType()
				.equals(IMAGETOOLBARCOMPONENT_TYPE.ZOOM_IN)){
			parent.getSpreadCanvas().zoomIn();
		}else if(((ImageToolBarComponent)e.getSource()).getImageToolBarComponentType()
				.equals(IMAGETOOLBARCOMPONENT_TYPE.ZOOM_OUT)){
			parent.getSpreadCanvas().zoomOut();
		}else if(((ImageToolBarComponent)e.getSource()).getImageToolBarComponentType()
				.equals(IMAGETOOLBARCOMPONENT_TYPE.ERASER)){
			Region region = UniSelectedState.getInstance().getSelectedRegion();
			new DeleteRegionAction(region).execute();
//			DefaultState.getInstance().setMode(EraserMode.getInstance());
			
			parent.repaint();
		}else if(((ImageToolBarComponent)e.getSource()).getImageToolBarComponentType()
				.equals(IMAGETOOLBARCOMPONENT_TYPE.SHOW_REGION)){
			String showRegionMode = (String) ((JComboBox) e.getSource()).getSelectedItem();
			if(showRegionMode == showregionarray[showAll]) {
				parent.getSpreadCanvas().getSpread().changeShowRegion(Sheet.SHOW_ALL_REGIONS);
			} else if(showRegionMode == showregionarray[showProtected]) {
				parent.getSpreadCanvas().getSpread().changeShowRegion(Sheet.SHOW_PROTECTED_REGIONS);
			} else if(showRegionMode == showregionarray[showNo]) {
				parent.getSpreadCanvas().getSpread().changeShowRegion(Sheet.SHOW_NO_REGIONS);
			}
			parent.repaint();
		}
	}

	public static void setModeListItem(String string) {
		modelist.setSelectedItem(string);
	}
	
	public static void setShowRegionItem(String string) {
		showRegion.setSelectedItem(string);
	}
	
	private class Select extends JButton implements ImageToolBarComponent {

		public Select(ImageIcon imageIcon) {
			super(imageIcon);
		}

		@Override
		public IMAGETOOLBARCOMPONENT_TYPE getImageToolBarComponentType() {
			return IMAGETOOLBARCOMPONENT_TYPE.SELECT;
		}		
	}
	
	private class Eraser extends JButton implements ImageToolBarComponent {

		public Eraser(ImageIcon imageIcon) {
			super(imageIcon);
		}

		@Override
		public IMAGETOOLBARCOMPONENT_TYPE getImageToolBarComponentType() {
			return IMAGETOOLBARCOMPONENT_TYPE.ERASER;
		}
	}
	
	private class ModeList extends JComboBox implements ImageToolBarComponent {

		public ModeList(String[] modelistarray) {
			super(modelistarray);
		}

		@Override
		public IMAGETOOLBARCOMPONENT_TYPE getImageToolBarComponentType() {
			return IMAGETOOLBARCOMPONENT_TYPE.MODE_LIST;
		}
	}

	private class ShowRegion extends JComboBox implements ImageToolBarComponent {

		public ShowRegion(String[] showregionarray) {
			super(showregionarray);
		}

		@Override
		public IMAGETOOLBARCOMPONENT_TYPE getImageToolBarComponentType() {
			return IMAGETOOLBARCOMPONENT_TYPE.SHOW_REGION;
		}
	}

	private class ZoomIn extends JButton implements ImageToolBarComponent {

		public ZoomIn(ImageIcon imageIcon) {
			super(imageIcon);
		}

		@Override
		public IMAGETOOLBARCOMPONENT_TYPE getImageToolBarComponentType() {
			return IMAGETOOLBARCOMPONENT_TYPE.ZOOM_IN;
		}
	}

	private class ZoomOut extends JButton implements ImageToolBarComponent {

		public ZoomOut(ImageIcon imageIcon) {
			super(imageIcon);
		}

		@Override
		public IMAGETOOLBARCOMPONENT_TYPE getImageToolBarComponentType() {
			return IMAGETOOLBARCOMPONENT_TYPE.ZOOM_OUT;
		}
	}

}
